#!/usr/bin/perl
# ------------------------------------------------------------------------------
# File Name:		check.pl
# Author:		Richard Mayhew - South Africa
# Date:			1999/11/01
# Version:		1.0
# Description:		This script will grab and sort host alerts from
#			netsaints logs (netsaint.log) and process the info
# Email:		netsaint@splash.co.za
# ------------------------------------------------------------------------------
# Copyright 1999 (c) Richard Mayhew
# Credits go to Ethan Galstad for coding NetSaint
# If any changes are made to this script, please mail me a copy of the
# changes :)
# License GPL
# ------------------------------------------------------------------------------
# Date		Author		Reason
# ----		------		------
# 1999/09/28	RM		Creation
# 1999/10/15	TP		Help with Calculation Subroutines
#
# ------------------------------------------------------------------------------

# -----------------------------------------------------------------[ Require ]--
require 5.004;
# --------------------------------------------------------------------[ Uses ]--
use strict;
# --------------------------------------------------------------[ Enviroment ]--
$ENV{PATH} = "/bin";
$ENV{BASH_ENV} = "";
$ENV{ENV} = "";
$|=1;
my $greplocation = "/bin/egrep";
my $countfile = "/usr/local/netsaint/report/counter";
my $data = "/usr/local/netsaint/var/netsaint.log";
my $database = "/usr/local/netsaint/report/database/";
my $localhosts = "/usr/local/netsaint/report/hosts";
my $localservices = "/usr/local/netsaint/report/services";
my $groupfile = "/usr/local/netsaint/report/group.cfg";
my $counter1 = 0;
my $counter2 = 0;

my ($user, $system, $cuser, $csystem) = times;

system("rm -f $localservices >/dev/null 2>&1");
system("rm -f $localhosts >/dev/null 2>&1");
system("rm -f $countfile >/dev/null 2>&1");

# --------------------------------------------------------[ CriticalHostData ]--
sub CriticalHostData
{
#This routine grabs all the information for HOSTS out of the netsaint.log file.
#It is then written to a file where it will be used later in the script.
#This function is to minimize the time for the script to pass through the data as
#the log file can become large.

	my @tDown = `$greplocation ';DOWN;|;UNREACHABLE;|;RECOVERY;' $data`;
	my $rowss = @tDown;
	my $x = 0;
	for($x=0;$x<$rowss;$x++) {

		if($tDown[$x] !~ /SERVICE/) {
			$tDown[$x] =~ s/\;UNREACHABLE;/;CRITICAL;/g; 
			$tDown[$x] =~ s/\]//g;
			$tDown[$x] =~ s/\[//g;
			$tDown[$x] =~ s/\;/ /g;
			my @data1 = split(' ',$tDown[$x]);
			
			open (HOSTS_FILE, ">>$localhosts");
			print HOSTS_FILE "$data1[0] $data1[3] $data1[4]\n";
			close (HOSTS_FILE);
		}
	}
}

# -----------------------------------------------------[ CriticalServiceData ]--
sub CriticalServiceData
{
#This routine grabs all the information for SERVICES of hosts out of the netsaint.log file. 
#It is then written to a file where it will be used later in the script. 
#This function is to minimize the time for the script to pass through the data as
#the log file can become large.

	my @tDown = `$greplocation ';CRITICAL;|;UNKNOWN;|;RECOVERY;' $data`;
	my $rowss = @tDown;
	my $x =0;
	for($x=0;$x<$rowss;$x++) {

		if($tDown[$x] !~ /HOST/) {
			$tDown[$x] =~ s/\;UNKNOWN;/;CRITICAL;/g;
			$tDown[$x] =~ s/\]//g;
			$tDown[$x] =~ s/\[//g;
			$tDown[$x] =~ s/\;/ /g;
		my @data1 = split(' ',$tDown[$x]);

			open (SERVICES_FILE, ">>$localservices");
			print SERVICES_FILE "$data1[0] $data1[3] $data1[4] $data1[5]\n";
			close (SERVICES_FILE);
		}
	}
}

# ----------------------------------------------------------------[ getGroup ]--
sub getGroup
{
#This routine checks the group file to see which group a particular
#host is in.

	my ($in_hostname) = @_;
	my $out_group = "";
	my $group ="";
	my $hosts ="";
	my @host_data = "";
	my $host ="";
	
	open(GRP_FILE,"<$groupfile") or die "Error Opening $groupfile File!\n";
	while (<GRP_FILE>) {
		chomp($_);
		($group,$hosts) = split(/\;/,$_);
		(@host_data) = split(/\,/,$hosts);
		foreach $host (@host_data) {
			if ($host eq "$in_hostname") {
				$out_group = $group;
			}
		}
	}
	close(GRP_FILE);
	return $out_group;
}

# -----------------------------------------------------------------[ logTime ]--
sub logTime
{
#This routine calculates the total downtime for a particular host with in the group.

	my ($in_group,$in_host,$in_time) = @_;
	my %info = ();
	my $time = 0;
	my $hst ="";
	my $tme ="";
	my $host ="";
	
	open(GRP_FILE,"<$database$in_group-Group") or die "Error Opening $in_group-Group File!\n";
	while (<GRP_FILE>) {
		chomp($_);
		($hst,$tme) = split(/\,/,$_);
		if ($hst eq "$in_host") {
			$tme = $in_time;
			$info{$hst} = $tme;

		} else {
			$info{$hst} = $tme;
		}
	}
	close(GRP_FILE);

	open(GRP_FILE,">$database$in_group-Group");
	foreach $host (sort keys(%info)) {
		print GRP_FILE "$host,$info{$host}\n";
	}
	close(GRP_FILE);
}

# ------------------------------------------------------------[ logtimeYear ]--
sub logTimeYear
{
#This routine calculates the total downtime for a host on a yearly basis.
        my ($in_month,$in_host,$in_time) = @_;
        my %info = ();
        my $time = 0;
       	my $mnth = "";
	my $tme = "";
 	my $month = "";	
	my $x = 0;	
	open(TIME_FILE,"<$database$in_host-Yearly") or die "Error Opening $database$in_host-Yearly File!\n";
        while (<TIME_FILE>) {
                chomp($_);
                ($mnth,$tme) = split(/\,/,$_);
                if ($mnth eq "$in_month") {
                        $tme = $in_time;
                        $info{$mnth} = $tme;

                } else {
                        $info{$mnth} = $tme;
                }
        }
        close(TIME_FILE);

	open(TIME_FILE,">$database$in_host-Yearly");
	foreach $month (keys(%info)) {
		print TIME_FILE "$month,$info{$month}\n";
	}
	close(TIME_FILE);
	&YearSort($in_host);
}
# ------------------------------------------------------------- [YearSort ]--
sub YearSort
#This routine sorts the yearly database files into the correct order.
{
my ($in_data) = @_;
my $data1 = "";
my $data2 = "";
my $data3 = "";
my $data4 = "";
my $data5 = "";
my $data6 = "";
my $data7 = "";
my $data8 = "";
my $data9 = "";
my $data10 = "";
my $data11 = "";
my $data12 = "";
my @Data = "";
my $rowss = "";
my $datafile = "";
my @Data = "";
my $rowss = "";

open(TIME_FILE,"$database$in_data-Yearly");
while (<TIME_FILE>) {
                $datafile .= $_;
        }
        close(TIME_FILE);
	@Data = split('\n',$datafile);
	$rowss = @Data;
       	my $x = 0; 
	for($x=0;$x<$rowss;$x++) {

if ($Data[$x] =~ /Jan/) {

$data1 = $Data[$x];
}

elsif ($Data[$x] =~ /Feb/) {

$data2 = $Data[$x];
}

elsif ($Data[$x] =~ /Mar/) {

$data3 = $Data[$x];
}

elsif ($Data[$x] =~ /Apr/) {

$data4 = $Data[$x];
}

elsif ($Data[$x] =~ /May/) {

$data5 = $Data[$x];
}

elsif ($Data[$x] =~ /Jun/) {

$data6 = $Data[$x];
}

elsif ($Data[$x] =~ /Jul/) {

$data7 = $Data[$x];
}

elsif ($Data[$x] =~ /Aug/) {

$data8 = $Data[$x];
}

elsif ($Data[$x] =~ /Sep/) {

$data9 = $Data[$x];
}

elsif ($Data[$x] =~ /Oct/) {

$data10 = $Data[$x];
}

elsif ($Data[$x] =~ /Nov/) {

$data11 = $Data[$x];
}

elsif ($Data[$x] =~ /Dec/) {

$data12 = $Data[$x];
}

}
open(TIME_FILE,">$database$in_data-Yearly");
print TIME_FILE "$data1\n";
print TIME_FILE "$data2\n";
print TIME_FILE "$data3\n";
print TIME_FILE "$data4\n";
print TIME_FILE "$data5\n";
print TIME_FILE "$data6\n";
print TIME_FILE "$data7\n";
print TIME_FILE "$data8\n";
print TIME_FILE "$data9\n";
print TIME_FILE "$data10\n";
print TIME_FILE "$data11\n";
print TIME_FILE "$data12\n";
close(TIME_FILE);
}

# ------------------------------------------------------------- [YearServiceSort ]--
sub YearServiceSort
#This routine sorts the yearly service database files into the correct order.
{
my ($in_data,$in_dataservice) = @_;

my $data1 = "";
my $data2 = "";
my $data3 = "";
my $data4 = "";
my $data5 = "";
my $data6 = "";
my $data7 = "";
my $data8 = "";
my $data9 = "";
my $data10 = "";
my $data11 = "";
my $data12 = "";
my $datafile = "";
my @Data = "";
my $rowss = "";

open(TIME_FILE,"$database$in_data-$in_dataservice-Yearly");
while (<TIME_FILE>) {
                $datafile .= $_;
        }
        close(TIME_FILE);
        @Data = split('\n',$datafile);
        $rowss = @Data;
        my $x = 0;
        for($x=0;$x<$rowss;$x++) {
if ($Data[$x] =~ /Jan/) {

$data1 = $Data[$x];
}

elsif ($Data[$x] =~ /Feb/) {

$data2 = $Data[$x];
}

elsif ($Data[$x] =~ /Mar/) {

$data3 = $Data[$x];
}

elsif ($Data[$x] =~ /Apr/) {

$data4 = $Data[$x];
}

elsif ($Data[$x] =~ /May/) {

$data5 = $Data[$x];
}

elsif ($Data[$x] =~ /Jun/) {

$data6 = $Data[$x];
}

elsif ($Data[$x] =~ /Jul/) {

$data7 = $Data[$x];
}

elsif ($Data[$x] =~ /Aug/) {

$data8 = $Data[$x];

} 
elsif ($Data[$x] =~ /Sep/) {

$data9 = $Data[$x];
}

elsif ($Data[$x] =~ /Oct/) {

$data10 = $Data[$x];
}

elsif ($Data[$x] =~ /Nov/) {

$data11 = $Data[$x];
}

elsif ($Data[$x] =~ /Dec/) {

$data12 = $Data[$x];
}

}
open(TIME_FILE,">$database$in_data-$in_dataservice-Yearly");
print TIME_FILE "$data1\n";
print TIME_FILE "$data2\n";
print TIME_FILE "$data3\n";
print TIME_FILE "$data4\n";
print TIME_FILE "$data5\n";
print TIME_FILE "$data6\n";
print TIME_FILE "$data7\n";
print TIME_FILE "$data8\n";
print TIME_FILE "$data9\n";
print TIME_FILE "$data10\n";
print TIME_FILE "$data11\n";
print TIME_FILE "$data12\n";
close(TIME_FILE);
}

# ----------------------------------------------------------[ logTimeServiceYear ]--
sub logTimeServiceYear
{
#This routine calculates the total downtime for a Hosts particular Service on a yearly basis.

        my ($in_month,$in_host,$in_service,$in_time) = @_;
        my %info = ();
        my $time = 0;
	my $mnth = "";
	my $tme = "";
	my $month = "";

        open(TIME_FILE,"<$database$in_host-$in_service-Yearly") or die "Error Opening $database$in_host-$in_service-Yearly File!\n";
        while (<TIME_FILE>) {
                chomp($_);
                ($mnth,$tme) = split(/\,/,$_);
                if ($mnth eq "$in_month") {
                        $tme = $in_time;
                        $info{$mnth} = $tme;

                } else {
                        $info{$mnth} = $tme;
                }
        }
        close(TIME_FILE);
        open(TIME_FILE,">$database$in_host-$in_service-Yearly");
        foreach $month (keys(%info)) {
                print TIME_FILE "$month,$info{$month}\n";
        }
        close(TIME_FILE);
	&YearServiceSort($in_host,$in_service);
}

# -----------------------------------------------------------[ logTimeService ]--
sub logTimeService
{
#This routine calculates the total downtime for a Hosts particular services.

        my ($in_host,$in_service,$in_time) = @_;
        my %info = ();
        my $time = 0;
	my $hst = "";
	my $host = "";
	my $tme = "";
        
	open(SRV_FILE,"<$database$in_host-Services") or die "Error Opening $database$in_host-Services File!\n";
        while (<SRV_FILE>) {
                chomp($_);
                ($hst,$tme) = split(/\,/,$_);
                if ($hst eq "$in_service") {
                        $tme = $in_time;
                        $info{$hst} = $tme;

                } else {
                        $info{$hst} = $tme;
                }
        }
        close(SRV_FILE);

        open(SRV_FILE,">$database$in_host-Services");
        foreach $host (sort keys(%info)) {
                print SRV_FILE "$host,$info{$host}\n";
        }
        close(SRV_FILE);
}

# ---------------------------------------------------------[ ProcessHostData ]--
sub ProcessHostData
{
#This routine process the host downtime data. It calculates the downtime using sub-routines and then writes the 
#data to the correct file in the database.

	my $host_file = "";
	my @hosts = ();
	my $log_file = "";
	my $log = "";
	my ($log_secs,$hostname,$status);
	my %logging = ();
	my $key = "";
	my $value = "";
	my $group_name = "";
	my %final = "";
	my $month = `/bin/date +%b`;
	$month =~ s/\n//g;
	
	open (LOG_FILE, "<$localhosts");
	while (<LOG_FILE>) {
		$log_file .= $_;
	}
	close(LOG_FILE);

	my @logs = split(/\n/,$log_file);

	foreach $log (@logs) {
		($log_secs,$hostname,$status) = split(/\s+/,$log);

		if (($status eq "DOWN") && (!$logging{$hostname})) {
			$logging{$hostname} = $log_secs;
			$counter1++;
		} elsif (($status eq "RECOVERY") && ($logging{$hostname})) {
			$final{$hostname} += ($log_secs - $logging{$hostname}) / 60;
			undef $logging{$hostname};
		}
	}

	while (($key,$value) = each(%final)) {

		$group_name = &getGroup($key);

		&logTime($group_name,$key,$value);

		&logTimeYear($month,$key,$value);
	}
}

# ------------------------------------------------------[ ProcessServiceData ]--
sub ProcessServiceData
{
#This routine process the service downtime data. It calculates the downtime using sub-routines and then writes the
#data to the correct file in the database.

        my $host_file = "";
        my @hosts = ();
        my $log_file = "";
        my $log = "";
        my ($log_secs,$hostname,$service,$status);
        my %final = ();
        my %logging = ();
	my $key = "";
	my $value = "";
	my $hostkey = "";
	my $servicekey = "";
	my $month = `/bin/date +%b`;
        $month =~ s/\n//g;
        
	open (LOG_FILE, "<$localservices");
        while (<LOG_FILE>) {
                $log_file .= $_;
        }
        close(LOG_FILE);

        my @logs = split(/\n/,$log_file);

        foreach $log (@logs) {
                ($log_secs,$hostname,$service,$status) = split(/\s+/,$log);

                if (($status eq "CRITICAL") && (!$logging{"$hostname $service"})) {
                        $logging{"$hostname $service"} = $log_secs;
			$counter2++;
                } elsif (($status eq "RECOVERY") && ($logging{"$hostname $service"})) {
                        $final{"$hostname $service"} += ($log_secs - $logging{"$hostname $service"}) / 60;
                        undef $logging{"$hostname $service"};
                }
        }

        while (($key,$value) = each(%final)) {


                ($hostkey,$servicekey) = split(' ',$key);

		&logTimeService($hostkey,$servicekey,$value);
		# Log yearly service time
		&logTimeServiceYear($month,$hostkey,$servicekey,$value);

        }
	
}
# -------------------------------------------------------------[ Counter ]--
sub counter
{
#This routine counts the number of entries used to generated the stats
open (COUNTER, ">$countfile");
print COUNTER "$counter1\n";
print COUNTER "$counter2\n";
close(COUNTER);
}

# --------------------------------------------------------------------[ MAIN ]--
MAIN:
{
#This is the main routine where all routines are called.

	&CriticalHostData();
	&CriticalServiceData();
	&ProcessHostData();
	&ProcessServiceData();
	&counter();
	
}

